@@ -6,8 +6,10 @@ from __future__ import print_function, absolute_import, division
 from ruamel.yaml.error import *  # NOQA
 from ruamel.yaml.nodes import *  # NOQA
 from ruamel.yaml.compat import text_type, binary_type, to_unicode, PY2, PY3, ordereddict
+from ruamel.yaml.compat import nprint, nprintf  # NOQA
 from ruamel.yaml.scalarstring import (
-    PreservedScalarString,
+    LiteralScalarString,
+    FoldedScalarString,
     SingleQuotedScalarString,
     DoubleQuotedScalarString,
 )
@@ -63,6 +65,7 @@ class BaseRepresenter(object):
         self.represented_objects = {}  # type: Dict[Any, Any]
         self.object_keeper = []  # type: List[Any]
         self.alias_key = None  # type: Optional[int]
+        self.sort_base_mapping_type_on_output = True
 
     @property
     def serializer(self):
@@ -149,7 +152,7 @@ class BaseRepresenter(object):
         if style is None:
             style = self.default_style
         comment = None
-        if style and style[0] == '|':
+        if style and style[0] in '|>':
             comment = getattr(value, 'comment', None)
             if comment:
                 comment = [None, [comment]]
@@ -207,10 +210,11 @@ class BaseRepresenter(object):
         best_style = True
         if hasattr(mapping, 'items'):
             mapping = list(mapping.items())
-            try:
-                mapping = sorted(mapping)
-            except TypeError:
-                pass
+            if self.sort_base_mapping_type_on_output:
+                try:
+                    mapping = sorted(mapping)
+                except TypeError:
+                    pass
         for item_key, item_value in mapping:
             node_key = self.represent_key(item_key)
             node_value = self.represent_data(item_value)
@@ -585,7 +589,11 @@ class Representer(SafeRepresenter):
         else:
             tag = u'tag:yaml.org,2002:python/object/apply:'
             newobj = False
-        function_name = u'%s.%s' % (function.__module__, function.__name__)
+        try:
+            function_name = u'%s.%s' % (function.__module__, function.__qualname__)
+        except AttributeError:
+            # probably PY2
+            function_name = u'%s.%s' % (function.__module__, function.__name__)
         if not args and not listitems and not dictitems and isinstance(state, dict) and newobj:
             return self.represent_mapping(
                 u'tag:yaml.org,2002:python/object:' + function_name, state
@@ -631,12 +639,14 @@ if PY2:
 
 Representer.add_multi_representer(object, Representer.represent_object)
 
+Representer.add_multi_representer(type, Representer.represent_name)
 
 from ruamel.yaml.comments import (
     CommentedMap,
     CommentedOrderedMap,
     CommentedSeq,
     CommentedKeySeq,
+    CommentedKeyMap,
     CommentedSet,
     comment_attrib,
     merge_attrib,
@@ -666,7 +676,7 @@ class RoundTripRepresenter(SafeRepresenter):
             return self.represent_scalar(u'tag:yaml.org,2002:null', u'null')
         return self.represent_scalar(u'tag:yaml.org,2002:null', "")
 
-    def represent_preserved_scalarstring(self, data):
+    def represent_literal_scalarstring(self, data):
         # type: (Any) -> Any
         tag = None
         style = '|'
@@ -675,6 +685,24 @@ class RoundTripRepresenter(SafeRepresenter):
         tag = u'tag:yaml.org,2002:str'
         return self.represent_scalar(tag, data, style=style)
 
+    represent_preserved_scalarstring = represent_literal_scalarstring
+
+    def represent_folded_scalarstring(self, data):
+        # type: (Any) -> Any
+        tag = None
+        style = '>'
+        for fold_pos in reversed(getattr(data, 'fold_pos', [])):
+            if (
+                data[fold_pos] == ' '
+                and (fold_pos > 0 and not data[fold_pos - 1].isspace())
+                and (fold_pos < len(data) and not data[fold_pos + 1].isspace())
+            ):
+                data = data[:fold_pos] + '\a' + data[fold_pos:]
+        if PY2 and not isinstance(data, unicode):
+            data = unicode(data, 'ascii')
+        tag = u'tag:yaml.org,2002:str'
+        return self.represent_scalar(tag, data, style=style)
+
     def represent_single_quoted_scalarstring(self, data):
         # type: (Any) -> Any
         tag = None
@@ -772,14 +800,13 @@ class RoundTripRepresenter(SafeRepresenter):
         elif data._exp is None:
             # no exponent, "normal" dot
             prec = data._prec
-            if prec < 1:
-                prec = 1
-            # print('dw2', data._width, prec)
             ms = data._m_sign if data._m_sign else ""
             # -1 for the dot
             value = u'{}{:0{}.{}f}'.format(
                 ms, abs(data), data._width - len(ms), data._width - prec - 1
             )
+            if prec == 0 or (prec == 1 and ms != ""):
+                value = value.replace(u'0.', u'.')
             while len(value) < data._width:
                 value += u'0'
         else:
@@ -870,22 +897,41 @@ class RoundTripRepresenter(SafeRepresenter):
             item_comments = {}
         for idx, item in enumerate(sequence):
             node_item = self.represent_data(item)
-            node_item.comment = item_comments.get(idx)
+            self.merge_comments(node_item, item_comments.get(idx))
             if not (isinstance(node_item, ScalarNode) and not node_item.style):
                 best_style = False
             value.append(node_item)
         if flow_style is None:
-            if self.default_flow_style is not None:
+            if len(sequence) != 0 and self.default_flow_style is not None:
                 node.flow_style = self.default_flow_style
             else:
                 node.flow_style = best_style
         return node
 
+    def merge_comments(self, node, comments):
+        # type: (Any, Any) -> Any
+        if comments is None:
+            assert hasattr(node, 'comment')
+            return node
+        if getattr(node, 'comment', None) is not None:
+            for idx, val in enumerate(comments):
+                if idx >= len(node.comment):
+                    continue
+                nc = node.comment[idx]
+                if nc is not None:
+                    assert val is None or val == nc
+                    comments[idx] = nc
+        node.comment = comments
+        return node
+
     def represent_key(self, data):
         # type: (Any) -> Any
         if isinstance(data, CommentedKeySeq):
             self.alias_key = None
             return self.represent_sequence(u'tag:yaml.org,2002:seq', data, flow_style=True)
+        if isinstance(data, CommentedKeyMap):
+            self.alias_key = None
+            return self.represent_mapping(u'tag:yaml.org,2002:map', data, flow_style=True)
         return SafeRepresenter.represent_key(self, data)
 
     def represent_mapping(self, tag, mapping, flow_style=None):
@@ -922,11 +968,13 @@ class RoundTripRepresenter(SafeRepresenter):
         except AttributeError:
             item_comments = {}
         merge_list = [m[1] for m in getattr(mapping, merge_attrib, [])]
+        item_count = 0
         if bool(merge_list):
             items = mapping.non_merged_items()
         else:
             items = mapping.items()
         for item_key, item_value in items:
+            item_count += 1
             node_key = self.represent_key(item_key)
             node_value = self.represent_data(item_value)
             item_comment = item_comments.get(item_key)
@@ -945,7 +993,7 @@ class RoundTripRepresenter(SafeRepresenter):
                 best_style = False
             value.append((node_key, node_value))
         if flow_style is None:
-            if self.default_flow_style is not None:
+            if ((item_count != 0) or bool(merge_list)) and self.default_flow_style is not None:
                 node.flow_style = self.default_flow_style
             else:
                 node.flow_style = best_style
@@ -1085,6 +1133,21 @@ class RoundTripRepresenter(SafeRepresenter):
             tag = u'tag:yaml.org,2002:map'
         return self.represent_mapping(tag, data)
 
+    def represent_list(self, data):
+        # type: (Any) -> Any
+        try:
+            t = data.tag.value
+        except AttributeError:
+            t = None
+        if t:
+            if t.startswith('!!'):
+                tag = 'tag:yaml.org,2002:' + t[2:]
+            else:
+                tag = t
+        else:
+            tag = u'tag:yaml.org,2002:seq'
+        return self.represent_sequence(tag, data)
+
     def represent_datetime(self, data):
         # type: (Any) -> Any
         inter = 'T' if data._yaml['t'] else ' '
@@ -1110,7 +1173,11 @@ class RoundTripRepresenter(SafeRepresenter):
 RoundTripRepresenter.add_representer(type(None), RoundTripRepresenter.represent_none)
 
 RoundTripRepresenter.add_representer(
-    PreservedScalarString, RoundTripRepresenter.represent_preserved_scalarstring
+    LiteralScalarString, RoundTripRepresenter.represent_literal_scalarstring
+)
+
+RoundTripRepresenter.add_representer(
+    FoldedScalarString, RoundTripRepresenter.represent_folded_scalarstring
 )
 
 RoundTripRepresenter.add_representer(
